﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Net.Sockets;
using System.Net;
using library;
using System.IO;
using System.Runtime.Serialization.Formatters.Binary;


namespace CommandSniffer {
    public enum Protocol {
        TCP = 6,
        UDP = 17,
        Unknown = -1
    };

    public partial class CommandSnifferForm : Form {
        private Socket mainSocket;
        private byte[] byteData = new byte[4096];
        private bool bContinueCapturing = false;
        private String bufPort = "";
        private List<Byte> list = new List<byte>();
        private Command command;
        private bool condition = false;
        private String name = "";
        private String sPort = "";
        private String sFlag = "";

        private delegate void AddTreeNode(TreeNode node, bool con);

        public CommandSnifferForm() {
            InitializeComponent();
        }

        private void btnStart_Click(object sender, EventArgs e) {
            if ((cmbInterfaces.Text == "") || (port.Text == "0")) {
                MessageBox.Show("Select an Interface and port.", "MJsniffer",
                    MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            try {
                if (!bContinueCapturing) {

                    btnStart.Text = "&Stop";
                    port.Enabled = false;
                    cmbInterfaces.Enabled = false;
                    bContinueCapturing = true;


                    mainSocket = new Socket(AddressFamily.InterNetwork,
                        SocketType.Raw, ProtocolType.IP);

                    mainSocket.Bind(new IPEndPoint(IPAddress.Parse(cmbInterfaces.Text), 0));

                    mainSocket.SetSocketOption(SocketOptionLevel.IP,
                                               SocketOptionName.HeaderIncluded,
                                               true);

                    byte[] byTrue = new byte[4] { 1, 0, 0, 0 };
                    byte[] byOut = new byte[4] { 1, 0, 0, 0 };

                    mainSocket.IOControl(IOControlCode.ReceiveAll, byTrue, byOut);

                    mainSocket.BeginReceive(byteData, 0, byteData.Length, SocketFlags.None,
                        new AsyncCallback(OnReceive), null);
                } else {
                    btnStart.Text = "&Start";
                    port.Enabled = true;
                    cmbInterfaces.Enabled = true;
                    bContinueCapturing = false;
                    mainSocket.Close();
                }
            } catch (Exception ex) {
                MessageBox.Show(ex.Message, "MJsniffer", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void OnReceive(IAsyncResult ar) {
            try {
                int nReceived = mainSocket.EndReceive(ar);

                ParseData(byteData, nReceived);

                if (bContinueCapturing) {
                    byteData = new byte[4096];

                    mainSocket.BeginReceive(byteData, 0, byteData.Length, SocketFlags.None,
                        new AsyncCallback(OnReceive), null);
                }
            } catch (ObjectDisposedException) {
            } catch (Exception ex) {
                MessageBox.Show(ex.Message, "MJsniffer", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void ParseData(byte[] byteData, int nReceived) {
            TreeNode rootNode = new TreeNode();
            IPHeader ipHeader = new IPHeader(byteData, nReceived);
            switch (ipHeader.ProtocolType) {
                case Protocol.TCP:

                    TCPHeader tcpHeader = new TCPHeader(ipHeader.Data,

                                                        ipHeader.MessageLength);

                    if (tcpHeader.DestinationPort.Equals(port.Text) || tcpHeader.SourcePort.Equals(port.Text)) {
                        /*TreeNode tcpNode = MakeTCPTreeNode(tcpHeader);
                        write = true;                       
                        if(tcpHeader.SourcePort.Equals(port.Text))
                            serv = true;
                        rootNode.Nodes.Add(tcpNode);*/
                        if (tcpHeader.Flags.Equals("0x11 (FIN, ACK)")) {
                            copyData(tcpHeader.Data, tcpHeader.MessageLength);
                            exeData();
                            tcpHeader = null;
                        }
                        if (tcpHeader.Flags.Equals("0x18 (PSH, ACK)")) {
                            if (bufPort.Equals("")) {
                                copyData(tcpHeader.Data, tcpHeader.MessageLength);
                                bufPort = tcpHeader.SourcePort;
                                sPort = tcpHeader.SourcePort;
                                sFlag = tcpHeader.Flags;
                                if (bufPort.Equals(port.Text)) {
                                    name = "KLIENT";
                                } else {
                                    name = "SERVER";
                                }
                            } else if (bufPort.Equals(tcpHeader.SourcePort)) {
                                copyData(tcpHeader.Data, tcpHeader.MessageLength);
                            } else {
                                exeData();
                                copyData(tcpHeader.Data, tcpHeader.MessageLength);
                                bufPort = tcpHeader.SourcePort;
                                if (bufPort.Equals(port.Text)) {
                                    name = "KLIENT";
                                } else {
                                    name = "SERVER";
                                }
                            }
                        }
                    }


                    break;

                case Protocol.Unknown:
                    break;
            }
            if (condition) {
                rootNode.Nodes.Add(MakeCommandNode(command));
                AddTreeNode addTreeNode = new AddTreeNode(OnAddTreeNode);

                rootNode.Text = name;

                treeView.Invoke(addTreeNode, new object[] { rootNode, condition });

            }
        }

        private TreeNode MakeTCPTreeNode(TCPHeader tcpHeader) {
            TreeNode tcpNode = new TreeNode();

            tcpNode.Text = "TCP";

            tcpNode.Nodes.Add("Source Port: " + tcpHeader.SourcePort);
            tcpNode.Nodes.Add("Destination Port: " + tcpHeader.DestinationPort);
            tcpNode.Nodes.Add("Sequence Number: " + tcpHeader.SequenceNumber);

            if (tcpHeader.AcknowledgementNumber != "")
                tcpNode.Nodes.Add("Acknowledgement Number: " + tcpHeader.AcknowledgementNumber);

            tcpNode.Nodes.Add("Header Length: " + tcpHeader.HeaderLength);
            tcpNode.Nodes.Add("Flags: " + tcpHeader.Flags);
            tcpNode.Nodes.Add("Window Size: " + tcpHeader.WindowSize);
            tcpNode.Nodes.Add("Checksum: " + tcpHeader.Checksum);

            if (tcpHeader.UrgentPointer != "")
                tcpNode.Nodes.Add("Urgent Pointer: " + tcpHeader.UrgentPointer);

            return tcpNode;
        }

        private TreeNode MakeCommandNode(Command command) {
            TreeNode commandNode = new TreeNode();

            commandNode.Text = "Command";

            commandNode.Nodes.Add("Type: " + command.getType());
            if (command.getSource() != null)
                commandNode.Nodes.Add("Source: " + command.getSource());
            if (command.getDest() != null)
                commandNode.Nodes.Add("Destination: " + command.getDest());
            if (command.getData() != null)
                commandNode.Nodes.Add("Data: " + command.getData());
            if (command.getUser() != null)
                commandNode.Nodes.Add(MakeUserNode(command.getUser()));
            if (command.getContacts() != null)
                commandNode.Nodes.Add(MakeContactsNode(command.getContacts()));
            if (command.getContact() != null)
                commandNode.Nodes.Add(MakeContactNode(command.getContact()));
            return commandNode;
        }

        private TreeNode MakeUserNode(User user) {
            TreeNode userNode = new TreeNode();
            userNode.Text = "User";
            if (user.getLogin() != null)
                userNode.Nodes.Add("Login: " + command.getUser().getLogin());
            if (user.getPwd() != null || user.getPwd() != "")
                userNode.Nodes.Add("Password: " + command.getUser().getPwd());
            if (user.getName() != null)
                userNode.Nodes.Add("Name: " + command.getUser().getName());
            if (user.getSurname() != null)
                userNode.Nodes.Add("Surname: " + command.getUser().getSurname());
            if (user.getAddress() != null)
                userNode.Nodes.Add("Address: " + command.getUser().getAddress());
            if (user.getTown() != null)
                userNode.Nodes.Add("Town: " + command.getUser().getTown());
            if (user.getState() != null)
                userNode.Nodes.Add("State: " + command.getUser().getState());
            return userNode;
        }

        private TreeNode MakeContactsNode(List<Contact> list) {
            TreeNode contactsNode = new TreeNode();
            contactsNode.Text = "Contacts";
            for (int i = 0; i < list.Count; i++) {
                contactsNode.Nodes.Add(MakeContactNode(list[i]));
            }
            return contactsNode;
        }

        private TreeNode MakeContactNode(Contact contact) {
            TreeNode contactNode = new TreeNode();
            contactNode.Text = "Contact";
            if (contact.getNickname() != null)
                contactNode.Nodes.Add("Nickname: " + contact.getNickname());
            if (contact.getName() != null)
                contactNode.Nodes.Add("Name: " + contact.getName());
            if (contact.getSurname() != null)
                contactNode.Nodes.Add("Surname: " + contact.getSurname());
            if (contact.getAddress() != null)
                contactNode.Nodes.Add("Address: " + contact.getAddress());
            if (contact.getTown() != null)
                contactNode.Nodes.Add("Town: " + contact.getTown());
            if (contact.getState() != null)
                contactNode.Nodes.Add("State: " + contact.getState());
            if (contact.getUserID() != null)
                contactNode.Nodes.Add("UserID: " + contact.getUserID());
            if (contact.getOnline())
                contactNode.Nodes.Add("Online: true");
            else
                contactNode.Nodes.Add("Online: false");
            return contactNode;

        }

        private void OnAddTreeNode(TreeNode node, bool con) {
            if (con) {
                treeView.Nodes.Add(node);
                condition = false;
            }
        }

        private void SnifferForm_Load(object sender, EventArgs e) {
            string strIP = null;

            IPHostEntry HosyEntry = Dns.GetHostEntry((Dns.GetHostName()));
            if (HosyEntry.AddressList.Length > 0) {
                foreach (IPAddress ip in HosyEntry.AddressList) {
                    strIP = ip.ToString();
                    cmbInterfaces.Items.Add(strIP);
                }
            }
        }

        private void SnifferForm_FormClosing(object sender, FormClosingEventArgs e) {
            if (bContinueCapturing) {
                mainSocket.Close();
            }
        }
        private void copyData(byte[] byteData, int nReceived) {
            for (int i = 0; i < nReceived; i++) {
                list.Add(byteData[i]);
            }
        }
        private void exeData() {
            byte[] array = new byte[list.Count];
            for (int i = 0; i < list.Count; i++) {
                array[i] = list[i];
            }
            MemoryStream ms = new MemoryStream();
            ms.Write(array, 0, array.Length);
            ms.Seek(0, 0);
            BinaryFormatter b = new BinaryFormatter();
            Object obj = b.Deserialize(ms);
            command = (Command)obj;
            ms.Close();
            list.Clear();
            condition = true;
        }
    }
}